package com.lamatek.servlets.google;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.lamatek.tags.google.GoogleMapEventHandler;
import com.lamatek.tags.google.GoogleMapTag;

/**
 * DefaultEventListenerServlet
 * 
 * This class is the parent class for all servlet based event listeners
 * for the &lt;googlemaps> JSP Taglibrary. It provides support for all the
 * events available. When declaring &lt;googlemaps:event> tags in your
 * map, pass the URL to your subclass of DefaultEventListenerServlet as
 * the url attribute.
 * 
 * Users extending this class should override the methods for the event
 * types they're interested in. It is also recommended that the user
 * first call super.processXXXEvent() for the method to ensure standard
 * map behaviour.
 * 
 * The standard process method is called after all event based centric methods
 * have been called and gives the user the opportunity to handle other non-event
 * based interactions in the same class. Using this method, it is quite possible
 * to cleanly handle the interactions for an entire application in one handler.
 * 
 * Users may also opt to use the JavaBean event listening. 
 * @see com.lamatek.beans.google.DefaultEventListenerBean
 * 
 * @author Tom Cole
 * @version 0.40
 */
public class DefaultEventListenerServlet extends HttpServlet implements GoogleMapEventHandler {

    /**
     * Overrides doGet from HttpServlet.
     * 
     * Users should <em>not</em> override this method. Rather override
     * the method that handles the event(s) you're interested in. You can
     * handle non-event related requests through the process() method.
     * 
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @throws ServletException
     * @throws IOException
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        String event = request.getParameter("event");
        if (event != null) {
            String mapName = request.getParameter("map");
            if (mapName != null) {
                Object o = null;
                o = request.getSession().getAttribute(mapName);
                if (o != null) {
                    GoogleMapTag map = (GoogleMapTag) o;
                    if (event.equals("click")) {
                        if (request.getParameter("component") == null) {
	                        double latitude = -181.0d;
	                        double longitude = -181.0d;
	                        try {
	                            latitude = Double.parseDouble(request.getParameter("latitude"));
	                            longitude = Double.parseDouble(request.getParameter("longitude"));
	                        }
	                        catch(Exception ex) {
	                            
	                        }
	                        if ((longitude > -181.0 && latitude > -181))
	                            processClickEvent(map, longitude, latitude);
                        }
                        else {
                            processOverlayClickEvent(map, request.getParameter("component"), request.getParameter("type"));
                        }
                    }
                    else if (event.equals("dblclick")) {
                        if (request.getParameter("component") == null) {
	                        double latitude = -181.0d;
	                        double longitude = -181.0d;
	                        try {
	                            latitude = Double.parseDouble(request.getParameter("latitude"));
	                            longitude = Double.parseDouble(request.getParameter("longitude"));
	                        }
	                        catch(Exception ex) {
	                            
	                        }
	                        if ((longitude > -181.0 && latitude > -181))
	                            processDoubleClickEvent(map, longitude, latitude);
                        }
                        else {
                            processOverlayDoubleClickEvent(map, request.getParameter("component"), request.getParameter("type"));
                        }
                    }
                    else if (event.equals("zoom")) {
                        int zoom = -1;
                        try {
                            zoom = Integer.parseInt(request.getParameter("zoom"));
                        }
                        catch(Exception ex) {
                            
                        }
                        if (zoom >= 0) 
                            processZoomEvent(map, zoom);
                    }
                    else if (event.equals("moveend")) {
                        double latitude = -181.0d;
                        double longitude = -181.0d;
                        try {
                            latitude = Double.parseDouble(request.getParameter("latitude"));
                            longitude = Double.parseDouble(request.getParameter("longitude"));
                        }
                        catch(Exception ex) {
                            
                        }
                        if ((longitude > -181.0 && latitude > -181))
                            processMoveEndEvent(map, longitude, latitude);
                    }
                    else if (event.equals("maptypechanged")) {
                        String type = request.getParameter("type");
                        if (type != null)
                            processMapTypeChangedEvent(map, type);
                    }
                    process(map, request);
                }
                else {
                    System.out.println("Unable to load map named " + mapName);
                }
            }
            else {
                System.out.println("Unable to locate target map name.");
            }
        }
        else {
            System.out.println("No event found to process.");
        }
    }
    /**
     * This method is called after all the event based methods have been called.
     * Developers can use this method to handle non-event based requests.
     * 
     * The default implementation is empty to there is no need to call
     * super.process().
     * 
     * @param map The GoogleMapTag that generated the event.
     * @param request The original ServletRequest.
     */
    public void process(GoogleMapTag map, ServletRequest request) {
        
    }
    /**
     * This method is called if a click event is detected that was generated
     * by an overlay (box, circle, marker, polygon, polyline).
     * 
     * The default implementation is empty so there is no need to call
     * super.processMarkerClickEvent().
     * 
     * @param map The GoogleMapTag parent of the marker clicked.
     * @param id The id of the overlay that was clicked.
     * @param type The type of overlay clicked (box, circle, marker, polygon, polyline)
     */
    public void processOverlayClickEvent(GoogleMapTag map, String id, String type) {
        
    }
    /**
     * This method is called when a click event is detected that was generated
     * by a GoogleMapTag (&lt;googlemaps:map>).
     * 
     * The default implementation of this method is empty so there is no need
     * to call super.processClickEvent().
     * 
     * @param map The GoogleMapTag that generated the event.
     * @param longitude The longitude (in decimal form) where the click occured.
     * @param latitude The latitude (in decimal form) where the click occured.
     */
    public void processClickEvent(GoogleMapTag map, double longitude, double latitude) {
        
    }
    /**
     * This method is called when a double-click event is detected that was
     * generated by an overlay (box, circle, marker, polygon, polyline).
     * 
     * The default implementation of this method is empty so there is no need
     * to call super.processMarkerDoubleClickEvent().
     * 
     * @param map The GoogleMapTag parent of the marker double-clicked.
     * @param id The id of the overlay double-clicked.
     * @param type The type of overlay double-clicked (box, circle, marker, polygon, polyline)
     */
    public void processOverlayDoubleClickEvent(GoogleMapTag map, String id, String type) {
        
    }
    /**
     * This method is called when a double-click event is detected that was
     * generated by a GoogleMapTag (&lt;googlemaps:map>).
     * 
     * The default implementation of this method is empty so there is no
     * need to call super.processDoubleClickEvent().
     * 
     * @param map The GoogleMapTag that generated the event.
     * @param longitude The longitude (in decimal form) where the double-click occured.
     * @param latitude The latitude (in decimal form) where the double-click occured. 
     */
    public void processDoubleClickEvent(GoogleMapTag map, double longitude, double latitude) {
        
    }
    /**
     * This method is called when a zoom event is detected.
     * 
     * The default implementation of this method simulates the default behaviour of
     * a Google Map and any subclasses should call super.processZoomEvent() to 
     * ensure consistent behaviour.
     * 
     * @param map The GoogleMapTag that generated the event.
     * @param zoom The new requested zoom level.
     */
    public void processZoomEvent(GoogleMapTag map, int zoom) {
        map.setZoom(zoom);
    }
    /**
     * This method is called when a moveend event is detected.
     * 
     * The default implementation of this method simulates the default behaviour of
     * a Google Map and any subclasses should call super.processMoveEndEvent() to 
     * ensure consistent behaviour.
     * 
     * @param map The GoogleMapTag that generated the event.
     * @param longitude The new center longitude (in decimal form).
     * @param latitude The new center latitude (in decimal form).
     */
    public void processMoveEndEvent(GoogleMapTag map, double longitude, double latitude) {
        map.setCenterLatitude(latitude);
        map.setCenterLongitude(longitude);
    }
    /**
     * This method is called when a maptypechanged event is detected.
     * 
     * The default implementation of this method simulates the default behaviour of
     * a Google Map and any subclasses should call super.processMapTypeChangedEvent() to 
     * ensure consistent behaviour.
     * 
     * @param map The GoogleMapTag that generated the event.
     * @param type The new requested map type (map | satellite | hybrid).
     */
    public void processMapTypeChangedEvent(GoogleMapTag map, String type) {
        map.setType(type);
    }
}
